@@ -44,7 +44,6 @@ import pyatspi
 from . import braille_generator
 from . import debug
 from . import event_manager
-from . import flat_review
 from . import formatting
 from . import label_inference
 from . import keybindings
@@ -118,11 +117,8 @@ class Script:
         self.eventCache = {}
         self.whereAmI = self.getWhereAmI()
         self.spellcheck = self.getSpellCheck()
-        self.voices = settings.voices
         self.tutorialGenerator = self.getTutorialGenerator()
 
-        self.flatReviewContextClass = flat_review.Context
-
         self.findCommandRun = False
         self._lastCommandWasStructNav = False
 
@@ -159,25 +155,6 @@ class Script:
 
         return keybindings.KeyBindings()
 
-    def getKeyBindingsForInputHandler(self, inputEventHandler):
-        """ Returns a KeyBindings object with the list of KeyBindings that
-        matche the passed inputEventHandler as argument (at least the
-        inputEventHandler that has the same handler function)
-
-        Arguments:
-        - inputEventHandler: an instance of input_event.InputEventHandler
-
-        Returns an instance of keybindings.KeyBindings populated with
-        keybindings.KeyBinding instances that match the inputEventHandler.
-        """
-        matches = keybindings.KeyBindings()
-
-        for binding in self.keyBindings.keyBindings:
-            if inputEventHandler == binding.handler:
-                matches.add(binding)
-
-        return matches
-
     def getBrailleBindings(self):
         """Defines the braille bindings for this script.
 
@@ -186,20 +163,6 @@ class Script:
         """
         return {}
 
-    def getBrailleCommandsForInputHandler(self, inputEventHandler):
-        """Returns a list of BrlTTY commands (they're in braille.py) that
-        match the given inputEventHandler passed as argument.
-
-        Arguments:
-        - inputEventHandler: an instance of input_event.InputEventHandler
-
-        Returns a list (possibly empty) of BrlTTY commands (they're in
-        braille.py) that match the given inputEventHandler passed.
-        """
-        return [command
-                for command, handler in list(self.brailleBindings.items())
-                if inputEventHandler == handler]
-
     def getFormatting(self):
         """Returns the formatting strings for this script."""
         return formatting.Formatting(self)
@@ -373,7 +336,7 @@ class Script:
         # to the order in which they were added.  So...we need to do something
         # different here.  Logged as bugzilla bug 319781.]]]
         #
-        for key in list(self.listeners.keys()):
+        for key in self.listeners.keys():
             if event.type.startswith(key):
                 self.listeners[key](event)
 
@@ -469,7 +432,7 @@ class Script:
                 if consumes:
                     self._lastCommandWasStructNav = True
             else:
-                consumes = handler != None
+                consumes = handler is not None
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler \
@@ -478,55 +441,9 @@ class Script:
                 if consumes:
                     self._lastCommandWasStructNav = True
             else:
-                consumes = handler != None
+                consumes = handler is not None
         return consumes
 
-    def processKeyboardEvent(self, keyboardEvent):
-        """Processes the given keyboard event.
-
-        This method will primarily use the keybindings field of this
-        script instance see if this script has an interest in the
-        event.
-
-        NOTE: there is latent, but unsupported, logic for allowing
-        the user's user-settings.py file to extend and/or override
-        the keybindings for a script.
-
-        Arguments:
-        - keyboardEvent: an instance of input_event.KeyboardEvent
-        """
-
-        # We'll annotate the event with a reference to this script.
-        # This will allow external scripts to muck with the script
-        # instance if they wish.
-        #
-        keyboardEvent.script = self
-
-        # We'll let the user keybindings take precedence.  First, we'll
-        # check to see if they have keybindings specific for the particular
-        # application, then we'll check to see if they have any default
-        # bindings to use.
-        #
-        # [[[TODO: WDW - for performance, these bindings should probably
-        # be conflated at initialization time.]]]
-        #
-        user_bindings = None
-
-        user_bindings_map = settings.keyBindingsMap
-        if self.__module__ in user_bindings_map:
-            user_bindings = user_bindings_map[self.__module__]
-        elif "default" in user_bindings_map:
-            user_bindings = user_bindings_map["default"]
-
-        consumed = False
-        if user_bindings:
-            consumed = user_bindings.consumeKeyboardEvent(self,
-                                                          keyboardEvent)
-        if not consumed:
-            consumed = self.keyBindings.consumeKeyboardEvent(self,
-                                                             keyboardEvent)
-        return consumed
-
     def consumesBrailleEvent(self, brailleEvent):
         """Called when a key is pressed on the braille display.
 
@@ -616,24 +533,6 @@ class Script:
         """
         pass
 
-    def visualAppearanceChanged(self, event, obj):
-        """Called when the visual appearance of an object changes.
-        This method should not be called for objects whose visual
-        appearance changes solely because of focus -- setLocusOfFocus
-        is used for that.  Instead, it is intended mostly for objects
-        whose notional 'value' has changed, such as a checkbox
-        changing state, a progress bar advancing, a slider moving,
-        text inserted, caret moved, etc.
-
-        The primary purpose of this method is to present the changed
-        information to the user.
-
-        Arguments:
-        - event: if not None, the Event that caused this to happen
-        - obj: the Accessible whose visual appearance changed.
-        """
-        pass
-
     def isActivatableEvent(self, event):
         """Returns True if the given event is one that should cause this
         script to become the active script.  This is only a hint to
@@ -652,3 +551,6 @@ class Script:
     def deactivate(self):
         """Called when this script is deactivated."""
         pass
+
+    def getTransferableAttributes(self):
+        return {}
