@@ -698,14 +698,15 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
-        if self.utilities.isZombie(event.source):
+        role = event.source.getRole()
+
+        if self.utilities.isZombie(event.source) \
+           or role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
             comboBox = self.utilities.containingComboBox(event.source)
             if comboBox:
                 orca.setLocusOfFocus(event, comboBox, True)
                 return
 
-        role = event.source.getRole()
-
         # This seems to be something we inherit from Gtk+
         if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
             orca.setLocusOfFocus(event, event.source)
@@ -787,10 +788,6 @@ class Script(default.Script):
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
-        # We should present this in response to active-descendant-changed events
-        if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
-            return
-
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
@@ -827,6 +824,11 @@ class Script(default.Script):
             default.Script.onCheckedChanged(self, event)
             return
  
+        sourceWindow = self.utilities.topLevelObject(obj)
+        focusWindow = self.utilities.topLevelObject(orca_state.locusOfFocus)
+        if sourceWindow != focusWindow:
+            return
+
         # Announce when the toolbar buttons are toggled if we just toggled
         # them; not if we navigated to some text.
         weToggledIt = False
@@ -885,6 +887,10 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
+        if event.source != orca_state.locusOfFocus \
+           and event.source.getState().contains(pyatspi.STATE_FOCUSED):
+            orca.setLocusOfFocus(event, event.source, True)
+
         super().onTextSelectionChanged(event)
 
     def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
