@@ -49,30 +49,35 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateAncestors(obj, **args)
 
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_LINK:
-            return []
+        result = []
+        priorObj = args.get('priorObj')
+        if priorObj and self._script.utilities.inDocumentContent(priorObj):
+            priorDoc = self._script.utilities.getDocumentForObject(priorObj)
+            doc = self._script.utilities.getDocumentForObject(obj)
+            if priorDoc != doc and not self._script.utilities.getDocumentForObject(doc):
+                result = [super()._generateName(doc)]
 
-        if self._script.utilities.isLandmark(obj):
-            return []
-
-        if self._script.utilities.isMath(obj):
-            return []
+        if self._script.utilities.isLink(obj) \
+           or self._script.utilities.isLandmark(obj) \
+           or self._script.utilities.isMath(obj):
+            return result
 
         args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
                                pyatspi.ROLE_DOCUMENT_WEB,
                                pyatspi.ROLE_EMBEDDED,
                                pyatspi.ROLE_INTERNAL_FRAME,
-                               pyatspi.ROLE_FORM,
                                pyatspi.ROLE_MATH,
                                pyatspi.ROLE_MENU_BAR,
                                pyatspi.ROLE_TOOL_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_LABEL,
+                             pyatspi.ROLE_LINK,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
 
-        return super()._generateAncestors(obj, **args)
+        result.extend(super()._generateAncestors(obj, **args))
+
+        return result
 
     def _generateAllTextSelection(self, obj, **args):
         if self._script.utilities.isZombie(obj) \
@@ -142,7 +147,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return []
 
     def _generateLabelOrName(self, obj, **args):
-        if self._script.utilities.isTextBlockElement(obj):
+        if self._script.utilities.isTextBlockElement(obj) \
+           and not self._script.utilities.isLandmark(obj) \
+           and not self._script.utilities.isDPub(obj):
             return []
 
         if self._script.utilities.inDocumentContent(obj) and obj.name:
@@ -153,7 +160,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return super()._generateLabelOrName(obj, **args)
 
     def _generateName(self, obj, **args):
-        if self._script.utilities.isTextBlockElement(obj):
+        if self._script.utilities.isTextBlockElement(obj) \
+           and not self._script.utilities.isLandmark(obj) \
+           and not self._script.utilities.isDPub(obj):
             return []
 
         # TODO - JD: Once the formatting strings are vastly cleaned up
@@ -188,6 +197,18 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         return super()._generateNewNodeLevel(obj, **args)
 
+    def _generateLeaving(self, obj, **args):
+        if not args.get('leaving'):
+            return []
+
+        if self._script.utilities.inDocumentContent(obj) \
+           and not self._script.utilities.inDocumentContent(orca_state.locusOfFocus):
+            result = ['']
+            result.extend(self.voice(speech_generator.SYSTEM))
+            return result
+
+        return super()._generateLeaving(obj, **args)
+
     def _generateNewRadioButtonGroup(self, obj, **args):
         # TODO - JD: Looking at the default speech generator's method, this
         # is all kinds of broken. Until that can be sorted out, try to filter
@@ -231,6 +252,12 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         return super().getLocalizedRoleName(obj, **args)
 
+    def _generateRealActiveDescendantDisplayedText(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateRealActiveDescendantDisplayedText(obj, **args)
+
+        return self._generateDisplayedText(obj, **args)
+
     def _generateRoleName(self, obj, **args):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateRoleName(obj, **args)
@@ -245,14 +272,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return result
 
         role = args.get('role', obj.getRole())
-        force = args.get('force', False)
+        enabled, disabled = self._getEnabledAndDisabledContextRoles()
+        if role in disabled:
+            return []
 
+        force = args.get('force', False)
         start = args.get('startOffset')
         end = args.get('endOffset')
 
         if not force:
             doNotSpeak = [pyatspi.ROLE_FOOTER,
-                          pyatspi.ROLE_FORM,
                           pyatspi.ROLE_LABEL,
                           pyatspi.ROLE_MENU_ITEM,
                           pyatspi.ROLE_PARAGRAPH,
@@ -264,7 +293,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not force:
             doNotSpeak.append(pyatspi.ROLE_TABLE_CELL)
             doNotSpeak.append(pyatspi.ROLE_TEXT)
-            doNotSpeak.append('ROLE_STATIC')
+            doNotSpeak.append(pyatspi.ROLE_STATIC)
             if args.get('formatType', 'unfocused') != 'basicWhereAmI':
                 doNotSpeak.append(pyatspi.ROLE_LIST_ITEM)
                 doNotSpeak.append(pyatspi.ROLE_LIST)
@@ -308,7 +337,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(self.getLocalizedRoleName(obj, **args))
                 result.extend(acss)
 
-        elif role not in doNotSpeak:
+        elif role not in doNotSpeak and args.get('priorObj') != obj:
             result.append(self.getLocalizedRoleName(obj, **args))
             result.extend(acss)
 
@@ -405,9 +434,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if obj.getRole() in menuRoles:
             return super()._generatePositionInList(obj, **args)
 
-        if self._script.utilities.isTextBlockElement(obj):
-            return []
-
         if self._script.utilities.isEditableComboBox(obj):
             return []
 
@@ -458,17 +484,21 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             oldRole = self._overrideRole('default', args)
         elif self._script.utilities.isLink(obj):
             oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
-        elif self._script.utilities.isAnchor(obj):
-            oldRole = 'ROLE_STATIC'
-        elif self._script.utilities.treatAsDiv(obj):
+        elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
             oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
         else:
             oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
 
-        result.extend(super().generateSpeech(obj, **args))
-        result = list(filter(lambda x: x, result))
-        self._restoreRole(oldRole, args)
+        if not 'priorObj' in args:
+            args['priorObj'] = self._script.utilities.getPriorContext()[0]
 
+        if self._script.utilities.isLabellingContents(obj):
+            result = list(filter(lambda x: x, self.generateContext(obj, **args)))
+
+        if not result:
+            result = list(filter(lambda x: x, super().generateSpeech(obj, **args)))
+
+        self._restoreRole(oldRole, args)
         msg = "WEB: Speech generation for document object %s complete:" % obj
         debug.println(debug.LEVEL_INFO, msg)
         for element in result:
@@ -494,6 +524,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 index=i, total=len(contents), **args)
             if utterance and utterance[0]:
                 result.append(utterance)
+                args['priorObj'] = obj
 
         if not result:
             if self._script.inSayAll():
